#!/usr/bin/env python


import os

from sqlmap.settings import CSV_DIR


class Dump:
    """
    This class defines methods used to parse and output the results
    of SQL injection actions

    @author: Bernardo Damele and Daniele Bellucci
    """

    def __init__(self, args):
        self.args = args


    def string(self, header, data):
        if isinstance(data, (list, tuple, set)):
            self.lister(header, data)

            return

        if data:
            if "\n" in data:
                print "%s:\n---\n%s---\n" % (header, data)
            else:
                print "%s:    '%s'\n" % (header, data)
        else:
            print "%s:\tNone\n" % header


    def lister(self, header, elements):
        print "%s [%d]:" % (header, len(elements))

        try:
            elements = set(elements)
            elements = list(elements)
            elements.sort(key=lambda x: x.lower())
        except:
            pass

        for element in elements:
            if isinstance(element, str):
                print "[*] %s" % element
            elif isinstance(element, (list, tuple, set)):
                print "[*] " + ", ".join(e for e in element)
        print


    def passwordHashes(self, header, passwordHashes):
        print "%s:" % header

        users = passwordHashes.keys()
        users.sort(key=lambda x: x.lower())

        for user in users:
            passwords = passwordHashes[user]

            print "[*] %s [%d]:" % (user, len(passwords))

            for password in passwords:
                print "    password hash: %s" % password
        print


    def dbTables(self, dbTables):
        maxlength = 0

        for tables in dbTables.values():
            for table in tables:
                maxlength = max(maxlength, len(table))

        lines = "-" * (int(maxlength) + 2)

        for db, tables in dbTables.items():
            tables.sort(key=lambda x: x.lower())

            print "Database: %s" % db

            if len(tables) == 1:
                print "[1 table]"
            else:
                print "[%d tables]" % len(tables)

            print "+%s+" % lines

            for table in tables:
                blank = " " * (maxlength - len(table))
                print  "| %s%s |\n" % (table, blank),

            print "+%s+\n" % lines


    def dbTableColumns(self, tableColumns):
        for db, tables in tableColumns.items():
            if not db:
                db = "All"

            for table, columns in tables.items():
                maxlength1 = 0
                maxlength2 = 0

                colList = columns.keys()
                colList.sort(key=lambda x: x.lower())

                for column in colList:
                    colType = columns[column]
                    maxlength1 = max(maxlength1, len(column))
                    maxlength2 = max(maxlength2, len(colType))

                maxlength1 = max(maxlength1, len("COLUMN"))
                maxlength2 = max(maxlength2, len("TYPE"))
                lines1 = "-" * (int(maxlength1) + 2)
                lines2 = "-" * (int(maxlength2) + 2)

                print "Database: %s\nTable: %s" % (db, table)

                if len(columns) == 1:
                    print "[1 column]"
                else:
                    print "[%d columns]" % len(columns)

                print "+%s+%s+" % (lines1, lines2)

                blank1 = " " * (maxlength1 - len("COLUMN"))
                blank2 = " " * (maxlength2 - len("TYPE"))

                print "| Column%s" % blank1,
                print "| Type%s |" % blank2
                print "+%s+%s+" % (lines1, lines2)

                for column in colList:
                    colType = columns[column]
                    blank1 = " " * (maxlength1 - len(column))
                    blank2 = " " * (maxlength2 - len(colType))
                    print "| %s%s" % (column, blank1),
                    print "| %s%s |" % (colType, blank2)

                print "+%s+%s+\n" % (lines1, lines2)


    def dbTableValues(self, tableValues):
        db = tableValues["__infos__"]["db"]
        if not db:
            db = "All"
        table = tableValues["__infos__"]["table"]

        if not self.args.googleDork:
            csvTablePath = "%s%s%s" % (self.args.csvPath, os.sep, db)

            if not os.path.isdir(csvTablePath):
                os.mkdir(csvTablePath)

            csvFile = file("%s%s%s.csv" % (csvTablePath, os.sep, table), "w")

        count = int(tableValues["__infos__"]["count"])
        separator = ""
        field = 1
        fields = len(tableValues) - 1

        columns = tableValues.keys()
        columns.sort(key=lambda x: x.lower())

        for column in columns:
            if column != "__infos__":
                info = tableValues[column]
                lines = "-" * (int(info["length"]) + 2)
                separator += "+%s" % lines

        separator += "+"
        print "Database: %s\nTable: %s" % (db, table)

        if count == 1:
            print "[1 entry]"
        else:
            print "[%d entries]" % count

        print separator

        for column in columns:
            if column != "__infos__":
                info = tableValues[column]
                maxlength = int(info["length"])
                blank = " " * (maxlength - len(column))
                print  "| %s%s" % (column, blank),

                if not self.args.googleDork and field == fields:
                    csvFile.write("\"%s\"" % column)
                else:
                    csvFile.write("\"%s\"," % column)

                field += 1

        print "|\n%s" % separator
        if not self.args.googleDork:
            csvFile.write("\n")

        for i in range(count):
            field = 1

            for column in columns:
                if column != "__infos__":
                    info = tableValues[column]
                    value = info["values"][i]
                    maxlength = int(info["length"])
                    blank = " " * (maxlength - len(value))
                    print  "| %s%s" % (value, blank),

                    if field == fields:
                        csvFile.write("\"%s\"" % value)
                    else:
                        csvFile.write("\"%s\"," % value)

                    field += 1

            print "|"
            if not self.args.googleDork:
                csvFile.write("\n")

        print "%s\n" % separator
        if not self.args.googleDork:
            csvFile.write("\n")
            csvFile.close()

