#!/usr/bin/env python


import re

from sqlmap.query import QueryMap
from sqlmap.settings import ORACLE_SYSTEM_DBS


class OracleMap(QueryMap):
    """
    This class defines Oracle methods

    @author: Bernardo Damele
    """


    def __init__(self, args):
        QueryMap.__init__(self, args, "Oracle")


    def unescape(self, expression):
        while True:
            index = expression.find("'")
            if index == -1:
                break

            firstIndex = index + 1
            index = expression[firstIndex:].find("'")

            if index == -1:
                raise Exception, "Unenclosed ' in '%s'" % expression

            lastIndex = firstIndex + index
            old = "'%s'" % expression[firstIndex:lastIndex]
            unescaped = "("

            for i in range(firstIndex, lastIndex):
                unescaped += "CHR(%d)" % (ord(expression[i]))
                if i < lastIndex - 1:
                    unescaped += "||"

            unescaped += ")"
            expression = expression.replace(old, unescaped)

        return expression


    def getFingerprint(self):
        if not self.args.exaustiveFp:
            return "Oracle"

        actVer = self.parseFp("Oracle", self.fingerprint)

        blank = " " * 16
        value = "active fingerprint: %s" % actVer

        if self.banner:
            banVer = re.search("^Oracle .*Release ([\d\.]+) ", self.banner)

            if banVer:
                banVer = banVer.groups()[0]
                banVer = self.parseFp("Oracle", [banVer])

                value += "\n%sbanner parsing fingerprint: %s" % (blank, banVer)

        self.passiveFuzzing()
        htmlParsed = self.getHtmlErrorFp()

        if htmlParsed:
            value += "\n%shtml error message fingerprint: %s" % (blank, htmlParsed)

        return value


    def checkDbms(self):
        if self.args.dbms == "oracle":
            self.setDbms("Oracle")

            if not self.args.exaustiveFp:
                return True

        logMsg = "testing Oracle"
        self.logger.info(logMsg)

        query = "LENGTH(SYSDATE)"
        sysdate = self.getValue(query)

        if sysdate and int(sysdate) > 0:
            logMsg = "confirming Oracle"
            self.logger.info(logMsg)

            query = "SELECT VERSION FROM SYS.PRODUCT_COMPONENT_VERSION WHERE ROWNUM=1"
            version = self.getValue(query)

            if not version:
                warnMsg = "the remote DMBS is not Oracle"
                self.logger.warn(warnMsg)

                return False

            self.setDbms("Oracle")

            if not self.args.exaustiveFp:
                return True

            if re.search("^11\.", version):
                self.fingerprint = ["11i"]
            elif re.search("^10\.", version):
                self.fingerprint = ["10g"]
            elif re.search("^9\.", version):
                self.fingerprint = ["9i"]
            elif re.search("^8\.", version):
                self.fingerprint = ["8i"]

            if self.args.getBanner:
                self.banner = self.getValue("SELECT banner FROM v$version WHERE ROWNUM=1")

            return True
        else:
            warnMsg = "the remote DMBS is not Oracle"
            self.logger.warn(warnMsg)

            return False


    def getDbs(self):
        warnMsg = "this plugin can not enumerate databases"
        self.logger.warn(warnMsg)

        return []

