#!/usr/bin/env python


import random
import re

from sqlmap.query import QueryMap


class PostgreSQLMap(QueryMap):
    """
    This class defines PostgreSQL methods

    @author: Bernardo Damele and Daniele Bellucci
    """

    def __init__(self, args):
        QueryMap.__init__(self, args, "PostgreSQL")


    def unescape(self, expression):
        while True:
            index = expression.find("'")
            if index == -1:
                break

            firstIndex = index + 1
            index = expression[firstIndex:].find("'")

            if index == -1:
                raise Exception, "Unenclosed ' in '%s'" % expression

            lastIndex = firstIndex + index
            old = "'%s'" % expression[firstIndex:lastIndex]
            unescaped = "("

            for i in range(firstIndex, lastIndex):
                unescaped += "CHR(%d)" % (ord(expression[i]))
                if i < lastIndex - 1:
                    unescaped += "||"

            unescaped += ")"
            expression = expression.replace(old, unescaped)

        return expression


    def getFingerprint(self):
        if not self.args.exaustiveFp:
            return "PostgreSQL"

        actVer = self.parseFp("PostgreSQL", self.fingerprint)

        blank = " " * 16
        value = "active fingerprint: %s" % actVer

        if self.banner:
            banVer = re.search("^PostgreSQL ([\d\.]+)", self.banner)
            banVer = banVer.groups()[0]
            banVer = self.parseFp("PostgreSQL", [banVer])

            value += "\n%sbanner parsing fingerprint: %s" % (blank, banVer)

        self.passiveFuzzing()
        htmlParsed = self.getHtmlErrorFp()

        if htmlParsed:
            value += "\n%shtml error message fingerprint: %s" % (blank, htmlParsed)

        return value


    def checkDbms(self):
        if self.args.dbms in ("postgresql", "postgres", "pgsql"):
            self.setDbms("PostgreSQL")

            if not self.args.exaustiveFp:
                return True

        logMsg = "testing PostgreSQL"
        self.logger.info(logMsg)

        randInt = str(random.randint(1, 9))
        query = "COALESCE(%s, NULL)" % randInt

        if self.getValue(query) == randInt:
            logMsg = "confirming PostgreSQL"
            self.logger.info(logMsg)

            query = "LENGTH('%s')" % randInt

            if not self.getValue(query) == "1":
                warnMsg = "the remote DMBS is not PostgreSQL"
                self.logger.warn(warnMsg)

                return False

            self.setDbms("PostgreSQL")

            if not self.args.exaustiveFp:
                return True

            if self.getValue("SUBSTR(TRANSACTION_TIMESTAMP(), 1, 1)") == "2":
                self.fingerprint = [">= 8.2.0"]
            elif self.getValue("GREATEST(5, 9, 1)") == "9":
                self.fingerprint = [">= 8.1.0", "< 8.2.0"]
            elif self.getValue("WIDTH_BUCKET(5.35, 0.024, 10.06, 5)") == "3":
                self.fingerprint = [">= 8.0.0", "< 8.1.0"]
            elif self.getValue("SUBSTR(MD5('sqlmap'), 1, 1)"):
                self.fingerprint = [">= 7.4.0", "< 8.0.0"]
            elif self.getValue("SUBSTR(CURRENT_SCHEMA(), 1, 1)") == "p":
                self.fingerprint = [">= 7.3.0", "< 7.4.0"]
            elif self.getValue("BIT_LENGTH(1)") == "8":
                self.fingerprint = [">= 7.2.0", "< 7.3.0"]
            elif self.getValue("SUBSTR(QUOTE_LITERAL('a'), 2, 1)") == "a":
                self.fingerprint = [">= 7.1.0", "< 7.2.0"]
            elif self.getValue("POW(2, 3)") == "8":
                self.fingerprint = [">= 7.0.0", "< 7.1.0"]
            elif self.getValue("MAX('a')") == "a":
                self.fingerprint = [">= 6.5.0", "< 6.5.3"]
            elif re.search("([\d\.]+)", self.getValue("SUBSTR(VERSION(), 12, 5)")):
                self.fingerprint = [">= 6.4.0", "< 6.5.0"]
            elif self.getValue("SUBSTR(CURRENT_DATE, 1, 1)") == "2":
                self.fingerprint = [">= 6.3.0", "< 6.4.0"]
            elif self.getValue("SUBSTRING('sqlmap', 1, 1)") == "s":
                self.fingerprint = [">= 6.2.0", "< 6.3.0"]
            else:
                self.fingerprint = ["< 6.2.0"]

            if self.args.getBanner:
                self.banner = self.getValue("VERSION()")

            return True
        else:
            warnMsg = "the remote DMBS is not PostgreSQL"
            self.logger.warn(warnMsg)

            return False

