#!/usr/bin/env python


import logging


class Resume:
    """
    This class defines methods used to resume a blind SQL injection
    output value

    @author: Bernardo Damele
    """

    def __init__(self, args):
        self.args = args
        self.logger = logging.getLogger("sqlmapLog")


    def resumeValue(self, expression, payload):
        """
        This method is called by request.getValue() to resume part or
        entire output of a blind SQL injection query output
        """

        resumedValue = self.args.resumedQueries[self.args.url][expression]
        resumedValueReplaced = resumedValue.replace("__NEWLINE__", "\n").replace("__TAB__", "\t")

        if resumedValueReplaced[-1] == "]":
            resumedValueReplaced = resumedValueReplaced[:-1]
 
            logMsg  = "read from file '%s': " % self.args.outputFile
            logMsg += "%s" % resumedValueReplaced.replace("__START__", "").replace("__STOP__", "").split("\n")[0]
            self.logger.info(logMsg)

            return resumedValueReplaced

        if not payload:
            return None

        expressionUnescaped, _, secondDbStr, method = self.queryVars(expression)
        _, length, regExpr = self.queryLength(expression, payload)

        if not length:
            return None

        if len(resumedValueReplaced) == int(length):
            logMsg  = "read from file '%s': " % self.args.outputFile
            logMsg += "%s" % resumedValueReplaced.split("\n")[0]
            self.logger.info(logMsg)

            return resumedValueReplaced
        elif len(resumedValueReplaced) < int(length):
            logMsg  = "resumed from file '%s': " % self.args.outputFile
            logMsg += "%s..." % resumedValueReplaced.split("\n")[0]
            self.logger.info(logMsg)

            if self.args.writeFile:
                self.args.writeFile.write("[%s][%s][%s][%s][%s" % (self.args.url, self.args.injPlace, self.args.parameters[self.args.injPlace], expression, resumedValue))
                self.args.writeFile.flush()

            if method == "SELECT":
                newExpr = expressionUnescaped.replace(regExpr, secondDbStr % (regExpr, len(resumedValueReplaced) + 1, int(length)), 1)
            else:
                newExpr = secondDbStr % (expressionUnescaped, len(resumedValueReplaced) + 1, int(length))

            missingCharsLength = int(length) - len(resumedValueReplaced)

            logMsg  = "retrieving pending %d query " % missingCharsLength
            logMsg += "output characters"
            self.logger.info(logMsg)

            count, finalValue = self.bisectionAlgorithm(payload, newExpr, length=missingCharsLength)

            if len(finalValue) != ( int(length) - len(resumedValueReplaced) ):
                warnMsg  = "the total length of the query is not "
                warnMsg += "right, sqlmap is going to retrieve the "
                warnMsg += "query value from the beginning now"
                self.logger.warn(warnMsg)

                return None

            return "%s%s" % (resumedValueReplaced, finalValue)

        return None
